#***********************************************************************************#
##Calculate the percent difference between adaptation and nominal strategies(yearly)#
#***********************************************************************************#

percentDiff <- function(myList, stats, myClimates, myStrats, myVariable, myCulvertName = NA){
  if(!is.na(myCulvertName)){
    myList <- myList[culvertName == myCulvertName,,]
    myList <- myList[,culvertName := NULL,]
  }
  stats <- stats[order(stats)]
  statsNames <- substr(rep(stats, length(myStrats)), 3, nchar(stats))
  statsNames <- statsNames[order(statsNames)]
  myColumns <- c(stats, "year", "climate", "strat")
  myList <- myList[variable == myVariable & climate %in% myClimates & strat %in% myStrats, myColumns, with = F]
  year <- max(myList$year)
  returnFrame <- array(NA, dim = c(length(myClimates), 
                                   length(myStrats)*length(stats), year), dimnames = list(myClimates, paste(myStrats, statsNames , sep = ""), 1:year))
  baseline <- vector("numeric", length = length(stats))
  for(i in 1:year){
    currentColumn <- 1
    for(j in 1:length(stats)){
      baseline[j] <- as.numeric(myList[year == i & strat == "normal" & climate == "normal", stats[j], with = F])
      for(l in 1:length(myStrats)){
        for(k in 1:length(myClimates)){
          # print(paste("j", j, "k", k, "l", l))
          returnFrame[k, currentColumn, i] <- (as.numeric(myList[year == i & strat == myStrats[l] & 
                                                       climate == myClimates[k], stats[j], with = F]) - baseline[j])/baseline[j]

        }
        currentColumn <- currentColumn + 1
      }
    }
  }
  return(returnFrame)
}

#*******************************************************#
#Plots results from percent difference function ********#
#*******************************************************#

plotDiff <- function(diffFrame, rowRange, colRange){
  new <- data.frame(diffFrame[rowRange, colRange,])
  myNew <- t(new)
  if(length(rowRange ==1)){
    strStuff <- data.table(cbind.data.frame(rep(dimnames(diffFrame)[[2]], nrow(myNew)), 1:dim(diffFrame)[3]))
  }else{
    strStuff <- strsplit(as.character(rownames(myNew)), ".", T)  
    strStuff <- lapply(strStuff, FUN = function(x) data.frame(t(x)))
    strStuff <- rbindlist(strStuff)
  }
  setnames(strStuff, c("stat", "year"))
  myNew <- cbind.data.frame(strStuff, myNew)
  myNew <- melt(myNew, id.vars = c(1,2))
  ggplot(data = myNew, aes(x = year, y = value, group = interaction(variable, stat))) + 
    geom_path(aes(color = stat)) + facet_grid(.~variable) + geom_hline(aes(yintercept = 0))
}

#*******************************************#
#Takes year frame and finds means by year***#
#*******************************************#

yearlyMeans <- function(yearFrame, strats, variable, addNormal = F){
  years <- unique(yearFrame$year)
  numStrats <- length(strats)
  meansFrame <- data.frame(matrix(nrow = length(years), ncol = 1 + numStrats))
  names(meansFrame) <- c("year", strats)
  normalCost <- vector(mode = "numeric", length = 3)
  if(addNormal){
    for(i in 1:numStrats){
      normalCost[i] <- mean(yearFrame$value[yearFrame$strat == strats[i] & yearFrame$variable == "SUM_totalNormal"])
    }
  }
  yearFrame <- yearFrame[yearFrame$variable == variable,]
  normalCost <- data.frame(t(normalCost))
  for(i in years){
    tempFrame <- yearFrame[yearFrame$year == i,]
    meansFrame[i,1] <- i
    for(k in 1:length(strats)){
      stratFrame <- tempFrame$value[tempFrame$strat == strats[k]]
      meansFrame[i,k + 1] <- mean(stratFrame) + normalCost[k]
    }
  }
  meansFrame <- melt(meansFrame,1)
  return(meansFrame)
}



#*******************************************************#
#returns a df with the mean, 10,90% and sd**************#
#*******************************************************#

statSummary <- function(meltStatDF, byCulvert = T, stats = NULL) {
  
  if(byCulvert) {
    meanList <- aggregate(value~culvertName, data = meltStatDF[meltStatDF$variable == "NPV_totalCost",], FUN = mean)
    tenList <- aggregate(value~culvertName, data = meltStatDF[meltStatDF$variable == "NPV_totalCost",], FUN = function(x) quantile(x, .10))
    nintyList <- aggregate(value~culvertName, data = meltStatDF[meltStatDF$variable == "NPV_totalCost",], FUN = function(x) quantile(x, .90))
    sdList <- aggregate(value~culvertName, data = meltStatDF[meltStatDF$variable == "NPV_totalCost",], FUN = function(x) sd(x))
    frame1 <- merge(x = meanList, y = tenList, by.x = "culvertName", by.y = "culvertName")
    frame2 <- merge(x = nintyList, y = sdList, by.x = "culvertName", by.y = "culvertName")
    finalFrame <- merge(x = frame1, y = frame2, by.x = "culvertName", by.y = "culvertName")
    colnames(finalFrame) <- c("culvertName", "mean", "x10", "x90", "SD")
    finalFrame[,2:5] <- prettyNum(finalFrame[,2:5], big.mark = ",", preserve.width = "none")
  }else{
    if(is.null(stats)){
      stats <- "SUM_totalCost"
    }
    myStrats <- levels(as.factor(meltStatDF$strat))
    myClimates <- levels(as.factor(meltStatDF$climate))
    for(i in myStrats){
      for(j in myClimates){
        for(k in stats){
          
          
        }
      }
    }
    
  }
  return(finalFrame)
}

#*******************************************************#
#Creates a histogram or cdf from big list***************#
#list must be as data.table, if a signle variable is****#
#specified function will return plots by cliamte bins***#
#*******************************************************#

bigListPlot <- function(myList, type, myVariable = NA, faceter = NA){
  if(!is.na(myVariable)){
    myList <- myList[variable == myVariable,]
    myList[, cuts := cut(myList[,get(faceter)], breaks = 5)]
    facetBy <- as.formula("~cuts")
    myScale <- "fixed"
    title <- paste(myVariable, faceter)
  }else{
    facetBy <- as.formula("~variable")
    myScale <- "free_x"
    title <- "All Variables"
    # myList[, value := value/1e6]
  }
  if(type == "histogram"){
    ggplot(myList, aes(x = value, color = strategy)) + geom_histogram(position = "identity", fill = alpha("red", 0)) +
       scale_color_manual(values = c("red", "green", "blue", "black")) + 
      facet_wrap(facetBy, scales = myScale) + ggtitle(paste("histogram of", title)) + ylab("Millions of Dollars")
  }else{
    ggplot(myList, aes(x = value, color = strategy)) + stat_ecdf() +
      facet_wrap(facetBy, scales = myScale) + ggtitle(paste("CDF of", title))
  }
}


#*****************************************************#
####Create PDF with faceted plots of different *******#
####Metrics for each culvert in list******************#
#******************************************************
plotGenerator <- function(meltStatDF, byCulvert = T, fileName, type){
  
  if(type == "histogram"){
    for (i in levels(factor(meltStatDF$culvertName))){
      currentDF <- meltStatDF[meltStatDF$culvertName == i,]
      print(ggplot(currentDF, aes(value)) + geom_histogram(aes(y=..count..)) + 
              facet_wrap(~variable, scales="free") + labs(title = i))
    }
  }
  else if(type == "cdf") {
    for (i in levels(factor(meltStatDF$culvertName))){
      currentDF <- meltStatDF[meltStatDF$culvertName == i,]
      print(ggplot(currentDF, aes(value)) + stat_ecdf() + 
              facet_wrap(~variable, scales="free") + labs(title = i))
    }
  }
}


#********************************************************#
#Creates 1 big list of multiple simulation runs**********#
#********************************************************#
listCombiner <- function(statList, simNames = NA){
  if(is.na(simNames)){
    for (i in 1:length(statList)){
      statList[[i]]$run <- paste0("run", i) 
    }
  }else{
    for (i in 1:length(statList)){
      statList[[i]]$run <- simNames[i] 
    }
  }
  statList <- data.frame(rbindlist(statList))
  return(statList)
}

#****************************************************#
#return list for production of CDFS******************#
#****************************************************#
compareSimulations <- function(bigStat, stat, myClimate, strategy){ 
  completeFrame <- bigStat
  tempList <- vector("list", length(stat))
  for(i in 1:length(stat)){
    tempList[[i]] <- completeFrame[completeFrame$variable == stat[i],]
  } 
  completeFrame <- rbindlist(tempList)
  tempList <- vector("list", length(myClimate))
  for(i in 1:length(myClimate)){
    tempList[[i]] <- completeFrame[completeFrame$strat == strategy[i] & completeFrame$climate == myClimate[i],]
  }
  completeFrame <- rbindlist(tempList)
  mean(completeFrame$value)
  completeFrame$run <- paste0(completeFrame$climate,"_",completeFrame$strat)
  
  return(completeFrame)
}

##***************************************##
#Creates melted list for analysis********##
#****************************************##
melter <- function(climates, strats, loadList = F, listType, folderName = ""){
  if(loadList){
    if(folderName != ""){
      folderName <- paste0(folderName, "/")
    }
    myList <- NA
    for(i in climates){
      listName <- paste0(i,"List")
      listNames <- NA
      tryCatch({
        loadList <- load(paste0(folderName, listName, ".RData"), envir = environment())
        listNames <- names(get(loadList))
        list2env(eval(parse(text = loadList)), envir = environment())
      }, error = function(e){
        listNames <- NA
        print(paste(i,"not found"))
      })
      myList <- c(listNames, myList)
    }
  }
  myList <- unique(na.omit(myList))
  meltList <- vector("list", length(climates) * length(strats))
  counter <- 1
  for(i in myList){
    tempFrame <- get(i)
    tempFrame <- melt(tempFrame, 1)
    tempFrame$climate <- substr(i, 0, regexpr("_", i)-1)
    tempFrame$strat <- substr(i, regexpr("_", i)+1, regexpr("Stat", i)-1)
    meltList[[counter]] <- tempFrame
    counter <- counter + 1
  }
  returnList <- (rbindlist(meltList))
  returnList$climate <- tolower(returnList$climate)
  returnList[,climate := as.factor(climate)]
  returnList[,strat := as.factor(strat)]
  return(returnList)
}

#*******************************************************#
#Creates melts of results maining yearly data***********#
#*******************************************************#
meltByYear <- function(climates, strats, loadList = F, meltBy, path = ""){
  meltList <- vector("list", length(climates) * length(strats))
  counter <- 1
  for(i in climates){ 
    for(j in strats){
      meltName <- paste0(i,"_",j)
      if(loadList){
        load(paste0(path, meltName, ".RData"))
      }
      tempFrame <- eval(parse(text = meltName))
      tempFrame$floodReplace <- ifelse(tempFrame$floodDamage > 0 & tempFrame$replacement == 1, 1, 0)
      tempFrame <- data.table(tempFrame)
      if(meltBy == "culvert"){
        tempFrame <- melt(data = tempFrame, id.vars = c(1,5), measure.vars = c(2:4, 6, 8, 9))
        groupingVars <- c("culvertName", "year", "variable")
      }else if(meltBy == "year"){
        tempFrame <- melt(data = tempFrame, id.vars = c(5,7), measure.vars = c(2:4, 6, 8, 9))
        tempFrame <- tempFrame[,.(value = sum(value)), by = .(year, variable, iteration)]
        groupingVars <- c("year", "variable")
      }
      tempFrame <- tempFrame[, .(
        myMean = mean(value),  
        myMax = max(value),  
        X98 = quantile(value, .98),
        mySD = sd(value)
      ), by = c(groupingVars)]
      tempFrame$climate <- tolower(i)
      tempFrame$strat <- j
      meltList[[counter]] <- tempFrame 
      counter <- counter + 1
      rm(list = meltName)
    }
  }
  returnList <- rbindlist(meltList)
  return(returnList)
}

#*******************************************************#
#*Plot year melts***************************************#
#*******************************************************#
plotMeltTS <- function(myList, myClimate, myStrat, myVariable, myColumn, byCol, facetCol = "", sMethod = "loess"){
  #myColumns <- c(myColumn, "year", "climate", "strat")
  test <- myList[variable == myVariable & climate %in% myClimate & strat %in% myStrat, ]
  z <- ggplot(test) + geom_smooth(aes(x = year, y = get(myColumn), color = get(byCol)), method = sMethod) + ggtitle(myVariable)
  if(facetCol != ""){
    z <- z + facet_grid(as.formula(paste(".~", facetCol)))
  }
  print(z)
  return(z)
}

#*******************************************************#
#**************Plots two year melts on same plot********#
#*******************************************************#
compareMelts <- function(list1, list2, myClimate, myStrat, myVariable, myColumn, byCol, sMethod = "loess"){
  list1[,run := "run1"]
  list2[,run := "run2"]
  comList <- rbind(list1, list2)
  return(plotMeltTS(comList, myClimate, myStrat, myVariable, myColumn, byCol, facetCol = "run", sMethod))
}

#*******************************************************#
#Loads saved melts from sensitivty analys simulations***#
#*******************************************************#
loadMelts <- function(simulationName){
  fileNames <- list.files()
  fileNames <- fileNames[grep(simulationName, fileNames)]
  for(i in fileNames){
    load(paste0(i, "/melts.RData"), envir = globalenv())
  }
}

#*******************************************************#
#Loads stat list from sensitivity analysis sims*********#
#*******************************************************#
loadStatList <- function(simulationName, myClimate, myStrat){
  fileNames <- list.files()
  fileNames <- fileNames[grep(simulationName, fileNames)]
  initialWD <- getwd()
  fullList <- vector("list", length(fileNames))
  for(i in 1:length(fileNames)){
    fullList[[i]] <- melter(myClimate, myStrat, T, "Stat", fileNames[i])
    fullList[[i]][, "sensValue"] <- 
      substr(fileNames[i], nchar(simulationName) + 1, nchar(fileNames[i]))
  }
  fullList <- rbindlist(fullList)
  fullList[, sensValue := as.factor(sensValue)]
  return(fullList)
}

#*******************************************************#
#Creates table with descriptive stats of multiple sims**#
#*******************************************************#
compareSimulations <- function(simulationName, myClimate, sensValues, myStrats, myVariable){
  fileNames <- list.files()
  fileNames <- fileNames[grep(simulationName, fileNames)]
  resultList <- data.frame(matrix(NA, nrow = length(fileNames) * length(myClimate) * length(myStrats), ncol = 8))
  names(resultList) <- c("climate", "strat", "senValue", "myMean", "qfive", "qT_five", "qS_five", "qN_five")
  counter <- 1
  for(i in 1:length(fileNames)){
    for(j in 1:length(myClimate)){
      listName <- load(paste0(fileNames[i],"/", myClimate[j], "list.RData"))
      currentList <- get(listName)
      for(l in 1:length(myStrats)){
        tempList <- currentList[[l]]
        tempList <- tempList[,get(myVariable)]
        quants <- quantile(tempList, c(.01, .25, .75, .95))
        resultList[counter, "climate"] <- myClimate[j]
        resultList[counter, "strat"] <- myStrats[l]
        resultList[counter, "senValue"] <- sensValues[i]
        resultList[counter, "myMean"] <- mean(tempList)
        for(k in 5:(length(quants) + 4)){
          resultList[counter, k] <- quants[k-4]
        }
        counter <- counter + 1
      }
    }
  }
  return(resultList)
}


statBoxPlotter <- function(statList, myVariable, colorCol, facetCol){
  statList <- statList[variable == myVariable,]
  ggplot(data = statList) + geom_boxplot(aes(get(colorCol), value)) + facet_grid(as.formula(paste(".~", facetCol)))
}

#*******************************************************#
#*Loads reasults from multiple frames into one dt*******#
#*******************************************************#
loadManyList <- function(folderNames, defaultValues){
  bigList <- vector("list", length(folderNames))
  for(i in 1:length(folderNames)){
    bigList[[i]] <- loadStatList(folderNames[i], c("high", "medium", "low", "normal", "variable"), c("normal", "upgrade"))
    bigList[[i]][,sensValue := as.numeric(as.character(sensValue))]
    setnames(bigList[[i]], "sensValue", folderNames[i])
  }
  bigList <- rbindlist(bigList, fill = T)
  for(i in 1:length(folderNames)){
    bigList[is.na(get(folderNames[i])),  folderNames[i]:=defaultValues[i]]
  }
  return(bigList)
}

#*******************************************************#
#***Loads stats from sens simulations into one dt*******#
#*******************************************************#
loadManyListSen <- function(folderNames){
  bigList <- vector("list", length(folderNames))
  for(i in 1:length(folderNames)){
    bigList[[i]] <- loadStatList(folderNames[i], c("high", "low", "normal", "variable"), c("normal", "upgrade", "EE", "anticipate"))
    bigList[[i]][,sensValue := as.numeric(as.character(sensValue))]
    bigList[[i]][, sensVariable := folderNames[i]]
  }
  bigList <- rbindlist(bigList, fill = T)
  return(bigList)
}


#***************************************#
#*Calculate and plot WSS****************#
#***************************************#
klusterPlot <- function(data, numClusters, title = ""){
  nk <- numClusters  
  x <- data
  wss=1:nk
  
  #Calculate WSS
  wss[1] <- (nrow(x)-1)*sum(apply(x,2,var))
  for (i in 2:nk){ 
    wss[i] <- sum(kmeans(x,centers=i, algorithm = "MacQueen", iter.max = 100)$withinss)
  }
  
  #Plot Results
  plot(1:nk, wss, type="b", xlab="Number of Clusters",
       ylab="Within groups sum of squares", main = paste(title, "Precip Clusters")) 
}

#***************************************#
#*Creates box plots from cluser data****#
#***************************************#
klusterBoxPlots <- function(data, numCenters, title = ""){
  x <- kmeans(klusterList, centers = numCenters, algorithm = "MacQueen", iter.max = 1000, nstart = 10)
  klusterList$cluster <- as.factor(x$cluster)
  hist(as.numeric(klusterList$cluster), main = title)
  klusterList <- melt(klusterList, id.vars = c("cluster"))
  ggplot(data = klusterList) + geom_boxplot(aes(y = value, x = cluster)) + 
    facet_wrap(~variable, scales = "free") + ggtitle(title)
}

#***************************************#
#*converts list from long to wide ******#
#***************************************#
matchList <- function(myList, myVariable, aggregate){
  tcList <- myList[variable == myVariable,]
  if(aggregate){
    tc_Up <- tcList[strategy == "upgrade", mean(value), by = .(emergencyF, upgradeC, upgradeF,
                                                               climateF, maxD)]
    tc_Norm <- tcList[strategy == "normal", mean(value), by = .(emergencyF, upgradeC, upgradeF,
                                                                climateF, maxD)]
    tc_EE <- tcList[strategy == "EE", mean(value), by = .(emergencyF, upgradeC, upgradeF,
                                                          climateF, maxD)]
    tc_Ant <- tcList[strategy == "anticipate", mean(value), by = .(emergencyF, upgradeC, upgradeF,
                                                                   climateF, maxD)]
    tcLong <- cbind(tc_Up, "norm" = tc_Norm$V1, "EE" = tc_EE$V1, "anticipate" = tc_Ant$V1)
    setnames(tcLong, "V1", "upgrade")
  }else{
    tcLong <- cbind(tcList[strategy == "normal"], 
                    "upgrade" = tcList[strategy == "upgrade", value], 
                    "EE" = tcList[strategy == "EE", value], "anticipate" = tcList[strategy == "anticipate", value])
    setnames(tcLong, "value", "norm")
    listCols <- c("emergencyF", "upgradeC", "upgradeF", "climateF", "maxD", "norm", "upgrade", "EE", "anticipate")
    if("culvertName" %in% names(tcLong)){
      listCols <- c(listCols, "culvertName")
    }
    tcLong <- tcLong[, listCols, with = F]
  }
  tcLong[, stratChoice := factor(apply(tcLong[, .(norm, upgrade, EE, anticipate)], 1, FUN = which.min), 
                                 levels = c(1:4), labels = c("Nominal", "Upgrade", "React", "Anticipate"))]
  setnames(tcLong, c("upgrade", "norm", "EE", "anticipate"), c("Upgrade", "Nominal", "React", "Anticipate"))
  return(tcLong)
}

#*********************************************#
#*Wrapper for writing to latex courtesy of SO*#
#*********************************************#
wrapLatex <- function(dat,fil, ...) {
  cat(c("\\documentclass{article}\n",
        "\\usepackage{booktabs}\n",
        "\\pagestyle{empty}\n",
        "\\begin{document}\n"), file=fil)
  print(dat, file=fil, append=TRUE, ...)
  cat("\\end{document}\n", file=fil, append=TRUE) 
}


#******************************************************#
#*Results from multinomial prediction with comparisons*#
#******************************************************#
multiPredResults <- function(model, originalData, prefix = "", newData = NA){
  if(any(is.na(newData))){
    newData <- originalData
  }
  longResults <- cbind(originalData, "predict" = predict(model, newdata = newData))
  levels(longResults$predict) <- levels(longResults$stratChoice)
  longResults[, "acc" := ifelse(stratChoice == predict, 1, 0)]
  stratColumn <- which(names(longResults) %in% "stratChoice")
  predColumn <- which(names(longResults) %in% "predict")
  
  longResults[, best := as.numeric(apply(longResults, 1, FUN = function(x) x[which(names(longResults) %in% x[stratColumn])]))]
  longResults[, predicted := as.numeric(apply(longResults, 1, FUN = function(x) x[which(names(longResults) %in% x[predColumn])]))]
  longResults[, c("diffBestN", "diffPredN") := list(Nominal - best, Nominal - predicted)]
  longResults[, c("diffBestU", "diffPredU") := list(Upgrade - best, Upgrade - predicted)]
  namesList <- c("predict", "acc", "diffBestN", "diffPredN", "diffBestU", "diffPredU")
  newNames <- paste0(prefix, namesList)
  setnames(longResults, namesList, newNames)
}

#******************************************************#
#*Results from multinomial prediction with comparisons*#
#*returned as percent difference from best and pred****#
#******************************************************#
multiPredResultsPer <- function(model, originalData, prefix = "", newData = NA){
  if(any(is.na(newData))){
    newData <- originalData
  }
  longResults <- cbind(originalData, "predict" = predict(model, newdata = newData))
  levels(longResults$predict) <- levels(longResults$stratChoice)
  longResults[, "acc" := ifelse(stratChoice == predict, 1, 0)]
  stratColumn <- which(names(longResults) %in% "stratChoice")
  predColumn <- which(names(longResults) %in% "predict")
  
  longResults[, best := as.numeric(apply(longResults, 1, FUN = function(x) x[which(names(longResults) %in% x[stratColumn])]))]
  longResults[, predicted := as.numeric(apply(longResults, 1, FUN = function(x) x[which(names(longResults) %in% x[predColumn])]))]
  longResults[, c("diffBestN", "diffPredN") := list((Nominal - best)/best, (Nominal - predicted)/predicted)]
  longResults[, c("diffBestU", "diffPredU") := list((Upgrade - best)/best, (Upgrade - predicted)/predicted)]
  longResults[, "diffPred" := (predicted - best)/best]
  namesList <- c("predict", "acc", "diffBestN", "diffPredN", "diffBestU", "diffPredU", "diffPred")
  newNames <- paste0(prefix, namesList)
  setnames(longResults, namesList, newNames)
}

#******************************************************#
#*summarizes results table by mean and 90th percentile*#
#******************************************************#
resultsTable <- function(results, bestColumns, predColumns, fun = "mean"){
  results[, predDiff := predicted - best]
  orgR <- results
  myFrame <- vector("list", length(levels(results$climateBin)))
  if(length(bestColumns) == 3){
    myStrats <- c("Nominal", "All_Upgrade", "Model_Prediction")
  }else{
    myStrats <- c("Nominal", "All_Upgrade", "Model_Prediction", "Climate_Pred")
  }
  
  for(i in 1:length(levels(results$climateBin))){  
    results <- results[climateBin == levels(results$climateBin)[i],]
    myBest <- do.call(fun, list(x = results$best))
    myPred <- do.call(fun, list(x = results$predicted))
    x90Best <- quantile(results$best, probs = .9)
    x90Pred <- quantile(results$predicted, probs = .9)
    myFrame[[i]] <- data.frame(Strategy = myStrats ,
  #                         Percent_Correct = c(results[stratChoice  %in%  c("Nominal", "Upgrade"),.N, by = stratChoice][,N], 
  #                           nrow(results[predicted == best,]))/nrow(results),
                          Mean_vs_Best = apply(results[, bestColumns, with = F], 2, fun) / myBest,
                          Mean_vs_Pred = c(apply(results[, predColumns, with = F], 2, fun), rep(0, length(bestColumns) - length(predColumns))) / myPred,
                          x90_vs_Best = apply(results[, bestColumns, with = F], 2, quantile, probs = .9) / x90Best,
                          x90_vs_Pred = c(apply(results[, predColumns, with = F], 2, quantile, probs = .9), rep(0, length(bestColumns) - length(predColumns))) / x90Pred,
                          climate = results[1,climateBin])
    results <- orgR
  }
  return(rbindlist(myFrame))
}

#******************************************************#
#**********simplified results table********************#
#******************************************************#

resultsTableSimp <- function(results, bestColumns, predColumns){
  results[, predDiff := predicted - best]
  myBest <- mean(results$best)
  myPred <- mean(results$predicted)
  x90Best <- quantile(results$best, probs = .9)
  x90Pred <- quantile(results$predicted, probs = .9)
  
  myFrame <- data.frame(Strategy = c("Nominal", "All_Upgrade", "Model_Prediction"),
                        #                         Percent_Correct = c(results[stratChoice  %in%  c("Nominal", "Upgrade"),.N, by = stratChoice][,N], 
                        #                           nrow(results[predicted == best,]))/nrow(results),
                        Mean_vs_Best = apply(results[, bestColumns, with = F], 2, mean) / myBest,
                        Mean_vs_Pred = c(apply(results[, predColumns, with = F], 2, mean), 0) / myPred,
                        x90_vs_Best = apply(results[, bestColumns, with = F], 2, quantile, probs = .9) / x90Best,
                        x90_vs_Pred = c(apply(results[, predColumns, with = F], 2, quantile, probs = .9),0) / x90Pred)
  return(myFrame)
}

#******************************************************************#
#*creates box plot from results denerated from multipredresultsper*#
#******************************************************************#
multiBox <- function(perResults, division = 1.5){
  perResults <- perResults[, .(climateF, CdiffBestN, CdiffBestU, CdiffPred, NCdiffPred)]
  perResults <- melt(perResults, 1)
  perResults[, climateD := ifelse(climateF <= division, "Low Change >1.5", "High Change <1.5")]
  p1 <- ggplot(data = perResults, aes(x = variable, y = value)) + geom_boxplot() + 
    facet_grid(.~climateD)
  p2 <- ggplot(data = perResults, aes(x = variable, y = value)) + geom_boxplot()
  multiplot(p2, p1, cols = 1)
  
}

reDoList <- function(myStrat, folderName){
  myFiles <- list.files(pattern = folderName)
  myStrat <- paste0("_", myStrat)
  lapply(myFiles, function(x){
    currentFiles <- list.files(path = x)
    currentFiles <- currentFiles[sapply(myStrat, grep, x = currentFiles)]
    myData <- lapply(currentFiles, function(y){
      load(paste0(x, "/", y), envir = globalenv())
    })
    myList <- lapply(myData, function(z)doByStatFrame(get(z)))
    names(myList) <- paste0(myData, "Stat")
    objectName <- paste0(substr(myData[[1]], 1, regexpr("_", myData[[1]]) - 1), "List")
    assign(objectName, myList)
    save(list = objectName, file = paste0(x, "/", objectName, ".RData"))
    # rm(list = as.character(myData), envir = globalenv())
  })
}


# Multiple plot function
#
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  library(grid)
  
  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)
  
  numPlots = length(plots)
  
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }
  
  if (numPlots==1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

shuffleCulverts <- function(aggCulvert, multiModel, NCmultiModel, climateBins, combos, isRandom = F){
  cl <- makeCluster(8)
  registerDoParallel(cl)
  aggCulvert[, climateBin := cut(aggCulvert$climateF, climateBins)]
  culverts <- unique(aggCulvert$culvertName)
  cols <- ncol(aggCulvert)
  
  tempIteration <- aggCulvert[1:8,]
  
  shuffleList <- foreach(i = levels(aggCulvert$climateBin), .packages = 'data.table') %dopar% {
    shuffleList <- vector("list", combos * length(culverts))
    tempAgg <- aggCulvert[climateBin == i,]
    counter <- 1
    for(k in 1:combos){
      for(j in 1:length(culverts)){
        tempIteration[j,] <- tempAgg[culvertName == culverts[j],][sample(.N, 1)] 
      }
      tempIteration$iteration <- counter
      shuffleList[[counter]] <- tempIteration
      counter <- counter + 1
    }
    rbindlist(shuffleList)
  }
  stopCluster(cl)
  shuffleList <- rbindlist(shuffleList)
  shuffleListR_orig <- multiPredResults(multiModel, shuffleList, "C")
  if(isRandom){
    x <- sample(c(T,F), nrow(shuffleListR_orig), replace = T)
    shuffleListR_orig[,predicted := ifelse(x, Nominal, Upgrade)]
    shuffleListReturn <- shuffleListR_orig[, .(Nominal =  sum(Nominal), Upgrade = sum(Upgrade), React = sum(React), 
                                               Anticipate = sum(Anticipate), best = sum(best), 
                                               predicted = sum(predicted), climateF = mean(climateF)), 
                                              by = .(iteration, climateBin)]
  }else{
    shuffleListRC <- multiPredResults(NCmultiModel, shuffleList, "C")
    shuffleListReturn <- cbind(shuffleListR_orig, climatePred = shuffleListRC$predicted, 
                          climateDiff = shuffleListRC$predicted - shuffleListRC$best)
    shuffleListReturn <- shuffleListReturn[, .(Nominal =  sum(Nominal), Upgrade = sum(Upgrade), React = sum(React), 
                                     Anticipate = sum(Anticipate), Cacc = mean(Cacc), best = sum(best), 
                                     predicted = sum(predicted), CdiffBestN = sum(CdiffBestN), 
                                     CdiffPredN = sum(CdiffPredN),CdiffBestU = sum(CdiffBestU), 
                                     CdiffPredU = sum(CdiffPredU), climateF = mean(climateF), 
                                     climatePred = sum(climatePred), climateDiff = sum(climateDiff)),
                                 by = .(iteration, climateBin)]
  }
  
    
  return(shuffleListReturn)
}

createError <- function(aggCulvert, summaryFunction, uncertain = c(.1, .3, .5)){
  myRange <- aggCulvert[, lapply(.SD, range), .SD = names(aggCulvertMed)[1:5]]
  myRange$climateF <- 0
  returnList <- vector("list", length(uncertain))
  for(i in 1:length(uncertain)){
    tempFrame <- copy(aggCulvert)
    
    rangeVector <- myRange[2] - myRange[1]
    rangeVector <- rangeVector * uncertain[i]
    error <- apply(rangeVector, 2, FUN = function(x) rnorm(20000,0,x))
    tempFrame[,1:5] <- (tempFrame[,1:5, with = F]) + error
    print("shufflingCulverts")
    tempFrame <- temp <- shuffleCulverts(tempFrame, longAggMultiNC, longAggMulti, 10, 1000)
    print("done shuffling")
    tempFrame <- tempFrame[, lapply(.SD, function(x) (x-best)/best), by = .(climateBin, climateF),
                                 .SDcols = c("predicted", "best")]
    tempFrame[, best := NULL]
    setnames(tempFrame, "predicted", paste0("Vertical Error ", uncertain[i]))
    tempFrame[, stat := summaryFunction]
    tempFrame <- melt(tempFrame, id.vars = c(1,2,4))
    returnList[[i]] <- tempFrame
  }
  return(rbindlist(returnList))
}