source("simulationFunctions.R")
load("eightSL.RData")
load("fiveSL.RData")
source("analysisFunctions.R")
library(snowfall)
library(biglm)
library(triangle)
library(microbenchmark)
library("ggplot2")
library(reshape2)
library("FinCal")
library("data.table")
#library("psych")
library(lmomco)
library(PearsonDS)
library(doBy)
library(extRemes)
library(evd)
library("nnet")
library("verification")
library("plyr")
library(doParallel)
library(foreach)
library(ggthemes)

##Set State Variables
startYear <- 1
endYear <- 100
currentYear <- startYear

#Discount Rate for NPV calcs
discount <- .03

#Increase in return level capacity (i.e. with an upgrade factor of 2 a 100 yr culvert will be changed to the 
#200 year flood in the upgrade and EE strategies)
upgradeFactor <- 2

#Initaial cost to upgrade times the amount the culvert is being upgraded, for example if the flow capacity is being increased
#by 20% and the orginal culvert cost $100 the new culvert will cost 100*.2*1.25
upgradeCost <- 2

#Increases in storm size for different climate scenarios, all increases are to the return interval of the storm
#increases are done by altering the return parameter
highFactor <- 2
medFactor <- 1.66
lowFactor <- 1.33

#Extra cost for replacing culverts in an emergency
emergencyFactor <- 1.5

#Flow increase a new culvert can withstand before being destroyed, i.e. with a maxDamage of .2 a culvert's
#Max flow can go up to 20% over before it will be destroyed
maxDamage <- .2

#Modifier for cost increase after the intial upgrade .5 means it will be 50% cheaper
postUpgradeFactor <- .5


##Read in culvert list and create a locked perm culvert list
culvertList <- read.csv("dataFiles/culvertList.csv", stringsAsFactors = F)
culvertList <<- culvertList[order(culvertList$culvertName),]
rownames(culvertList) <- NULL
permCulvertList <- read.csv("dataFiles/culvertList.csv", stringsAsFactors = F)
permCulvertList <- permCulvertList[order(permCulvertList$culvertName),]
rownames(permCulvertList) <- NULL
lockBinding("permCulvertList", globalenv())
culvertUpgraded <- cbind.data.frame("culvertName" = culvertList$culvertName, "upgraded" = F)


##Read in Road LIst
roadList <- read.csv("dataFiles/roadList.csv", stringsAsFactors = F)

#Create a cost list and dataframe to track cost number of rows = rows in culvertList
costList <- vector(mode = "list", length = (endYear - startYear + 1))
cost <- data.frame(matrix(NA, ncol = 7, nrow = nrow(culvertList)), stringsAsFactors = F)
colnames(cost) <- c("culvertName", "delayHours", "floodDamage", "construction", "year", "replacement", "iteration")
iterationNum <<- 1

climates <- c("high", "normal", "low")
strats <- c("normal", "upgrade", "anticipate", "EE")
# strats <- "anticipate"

anticipateAmount <- .1

startSnow <- function(cores, bigRand){
  sfInit(parallel = TRUE, cpus = cores)
  sfExport("roadList", "costOfDelay", "createCulvert", "createRoad", "doByStatFrame", "doByCulvert")
  sfExport("culvertCost", "culvertLife", "eventProcessorR", "getClimate")
  sfExport("replaceCulvert", "meltByYear")
  sfExport("culvertList", "culvertSimR", "doByYear")
  sfExport("iterationNum", "permCulvertList", "culvertUpgraded", "postUpgradeFactor")
  sfExport("startYear", "endYear", "costList", "culvertList", "cost", "currentYear", "createCostDF")
  sfExport("highFactor", "medFactor", "lowFactor", "getLocation", "discount", "anticipateAmount")
  sfExport("upgradeFactor", "upgradeCost", "highFactor", "medFactor", "lowFactor", "emergencyFactor", "maxDamage")
  if (!bigRand){
    sfExport("eightyYearSL", "fiftyYearSL", "culvertSim", "eventProcessor")
  }
  
  sfLibrary(evd)
  sfLibrary(extRemes)
  sfLibrary(triangle)
  sfLibrary(data.table)
  sfLibrary(PearsonDS)
  sfLibrary(FinCal)
  sfLibrary(reshape2)
  sfClusterSetupRNG()
  startYears <- NA
}