source("simulationFunctions.R")
#load("eightSL.RData")
#load("fiveSL.RData")
source("analysisFunctions.R")
library(snowfall)
library(biglm)
library(triangle)
library(microbenchmark)
library("ggplot2")
library(reshape2)
library("FinCal")
library("data.table")
#library("psych")
library(lmomco)
library(PearsonDS)
library(doBy)
library(extRemes)
library(evd)
library("nnet")
library("verification")
library("plyr")
library(ggthemes)
library(doParallel)
library(foreach)


##Set State Variables
startYear <- 1
endYear <- 100
currentYear <- startYear

#Discount Rate for NPV calcs
discount <- .03

#Increase in return level capacity (i.e. with an upgrade factor of 2 a 100 yr culvert will be changed to the 
#200 year flood in the upgrade and EE strategies)
upgradeFactorRange <- c(1.2, 3)
#Initaial cost to upgrade times the amount the culvert is being upgraded, for example if the flow capacity is being increased
#by 20% and the orginal culvert cost $100 the new culvert will cost 100*.2*1.25

upgradeCostRange <- c(1,3)
#Increases in storm size for different climate scenarios, all increases are to the return interval of the storm
#increases are done by altering the return parameter
highFactor <- 2
medFactor <- 1.66
lowFactor <- 1.33

#Extra cost for replacing culverts in an emergency

emergencyRange <- c(1, 2)
#Flow increase a new culvert can withstand before being destroyed, i.e. with a maxDamage of .2 a culvert's
#Max flow can go up to 20% over before it will be destroyed

maxDamageRange <- c(.1, .3)
#Modifier for cost increase after the intial upgrade .5 means it will be 50% cheaper

postUpgradeFactor <- .5
##Read in culvert list and create a locked perm culvert list
culvertList <- read.csv("dataFiles/culvertList.csv", stringsAsFactors = F)
culvertList <<- culvertList[order(culvertList$culvertName),]
rownames(culvertList) <- NULL
permCulvertList <- read.csv("dataFiles/culvertList.csv", stringsAsFactors = F)
permCulvertList <- permCulvertList[order(permCulvertList$culvertName),]
rownames(permCulvertList) <- NULL
lockBinding("permCulvertList", globalenv())
culvertUpgraded <- cbind.data.frame("culvertName" = culvertList$culvertName, "upgraded" = F)


##Read in Road LIst
roadList <- read.csv("dataFiles/roadList.csv", stringsAsFactors = F)

#Create a cost list and dataframe to track cost number of rows = rows in culvertList
costList <- vector(mode = "list", length = (endYear - startYear + 1))
cost <- data.frame(matrix(NA, ncol = 7, nrow = nrow(culvertList)), stringsAsFactors = F)
colnames(cost) <- c("culvertName", "delayHours", "floodDamage", "construction", "year", "replacement", "iteration")
iterationNum <<- 1

#climates <- c("High", "Medium", "Normal", "Low")
climates <- c("high", "normal", "low")
strats <- c("normal", "upgrade")
#strats <- c("normal", "upgrade", "anticipate", "EE")

sfInit(parallel=TRUE, cpus=8)
sfExport("roadList", "costOfDelay", "createCulvert", "createRoad", "doByStatFrame", "doByCulvert")
sfExport("culvertCost", "culvertLife", "eventProcessorR", "getClimate")
sfExport("replaceCulvert", "meltByYear")
sfExport("culvertList", "culvertSimR", "doByYear")
sfExport("iterationNum", "permCulvertList", "culvertUpgraded", "postUpgradeFactor")
sfExport("startYear", "endYear", "costList", "culvertList", "cost", "currentYear", "createCostDF")
sfExport("highFactor", "medFactor", "lowFactor", "getLocation", "discount")
sfLibrary(evd)
sfLibrary(extRemes)
sfLibrary(triangle)
sfLibrary(data.table)
sfLibrary(PearsonDS)
sfLibrary(FinCal)
sfLibrary(reshape2)
sfClusterSetupRNG()
startYears <- NA

#Run Simulation using Snowfall
#  i <- "high"
#  j <- "anticipate"
bigIT <- 250
#bigList <- vector("list", bigIT)
bigCulvert <- vector("list", bigIT)
climate <- "random"
ptm <- proc.time()
for(l in 1:bigIT){
  for(i in 1:nrow(culvertList)){
    if(culvertList$serviceLife[i] == 80){
      sampleFrame <- 59
    }else{
      sampleFrame <- 49
    }
    startYears[i] <- sample(1:sampleFrame, 1) * -1
  }
  emergencyFactor <- runif(1, emergencyRange[1], emergencyRange[2])
  maxDamage <- runif(1, maxDamageRange[1], maxDamageRange[2])
  #postUpgradeFactor <- runif(1, postUpgradeFactorRange[1], postUpgradeFactorRange[2])
  upgradeCost <- runif(1, upgradeCostRange[1], upgradeCostRange[2])
  upgradeFactor <- runif(1, upgradeFactorRange[1], upgradeFactorRange[2])
  randomFactor <- runif(1, .75, 3)
  culvertList$installYear <- startYears
  sfExport("emergencyFactor", "maxDamage", "upgradeCost", "upgradeFactor", "randomFactor", "culvertList")
  #currentList <- vector("list", length = length(strats))
  currentListCul <- vector("list", length = length(strats)) 
  jCount <- 0
  for(j in strats){
    jCount <- jCount + 1
    strategy <- j
    counter <- 1
    sfExport("climate", "strategy")
    ptm <- proc.time()
    result <- sfLapply(1:5000, snowWrapperR_lite)
    proc.time() - ptm
    #currentList[[jCount]] <- rbindlist(lapply(result, with, statList))
    temp <- rbindlist(result)
  }
  #bigList[[l]] <-  rbindlist(currentList)
  bigCulvert[[l]] <- rbindlist(currentListCul)
}
proc.time() - ptm


bigCulvert <- rbindlist(bigCulvert)

x <- as.numeric(temp[variable == "SUM_totalCost" & culvertName == "Dolores", value])
its <- 1000


estint=cumsum(as.numeric(x))/(1:1000)
esterr=sqrt(cumsum((as.numeric(x)-estint)^2))/(1:1000)
plot(estint, xlab="Mean and error range",type="l",lwd=
         + 2,ylim=mean(as.numeric(x))+20*c(-esterr[1000],esterr[1000]),ylab="")
lines(estint+2*esterr,col="gold",lwd=2)
lines(estint-2*esterr,col="gold",lwd=2)

