source("culvertPackages.R")


####Create BigList
load("bigCulvert.RData")
bigCulvert[, Iteration := unlist(lapply(1:1040000, FUN = rep, 80))]
bigList <- bigCulvert[climateF >= 1, .(value = sum(value)), by = .(Iteration, emergencyF, upgradeC, upgradeF, climateF, maxD, changeYear, variable, strategy)]
bigList_bak <- bigList

##Load Test Sets
load("bigTest.RData")
bigCulvertR[, Iteration := unlist(lapply(1:166400, FUN = rep, 80))]
bigListR <- bigCulvertR[climateF >= 1, .(value = sum(value)), by = .(Iteration, emergencyF, upgradeC, upgradeF, climateF, maxD, changeYear, variable, strategy)]

load("randomCulvert.RData")
currentListCul <- rbindlist(currentListCul)
currentListCul <- currentListCul[,.(value = sum(value)), by = .(Iteration, emergencyF, upgradeC, upgradeF, climateF, maxD, changeYear, variable, strategy)]

load("randomIndv.RData")
testIndv <- randomIndvCul[,.(norm = sum(normal), upgrade = sum(upgrade), EE = sum(EE), anticipate = sum(anticipate)),
                           by = .(emergencyF, upgradeC, upgradeF, climateF, maxD, variable)]
testIndv[, stratChoice := factor(apply(testIndv[, .(norm, upgrade, EE, anticipate)], 1, FUN = which.min), 
                                 levels = c(1:4), labels = c("Nominal", "Upgrade", "React", "Anticipate"))]

###Load Columns for Multi-Nom analysis

predColumns <- c("CdiffPredN", "CdiffPredU")
bestColumns <- c("CdiffBestN", "CdiffBestU", "predDiff")

predColumnsNC <- c("NCdiffPredN", "NCdiffPredU")
bestColumnsNC <- c("NCdiffBestN", "NCdiffBestU", "predDiff")


##Prep List for Multi
tcLongAgg <- matchList(bigList, "SUM_totalCost", T)
testLongAgg <- matchList(bigListR, "SUM_totalCost", T)



####One at a time Sens
# folderNames <- c("upgradeCost", "upgradeFactor", "climateFactor", "maxDamage", "emergencyFactor")
folderNames <- c("upgradeCost", "climateFactor", "maxDamage", "emergencyFactor")
bigSens <- loadManyListSen(folderNames)
bigMeans <- bigSens[,.(myMean = mean(value)), by = c("variable", "strat", "climate", "sensValue", "sensVariable")]
bigNine <- bigSens[,.(myNine = quantile(value, .9)), by = c("variable", "strat", "climate", "sensValue", "sensVariable")]
bigTen <- bigSens[,.(myTen = quantile(value, .1)), by = c("variable", "strat", "climate", "sensValue", "sensVariable")]
bigSD <- bigSens[,.(mySD = sd(value)), by = c("variable", "strat", "climate", "sensValue", "sensVariable")]
bigMeans <- cbind(bigMeans, "myNine" = bigNine$myNine, "myTen" = bigTen$myTen)
bigMeans <- cbind(bigMeans, "mySD" = bigSD$mySD)

####One-At-A-Time Data Prep
bigMeans$myMean <- bigMeans$myMean/1e6
bigMeans$climate <- mapvalues(bigMeans$climate, from = c("high", "low", "normal", "variable"), 
                              to = c("high rate of climate change", "low rate of climate change", "no climate change", "variable climate change"))

bigMeans$sensVariable <- mapvalues(bigMeans$sensVariable, 
                                   from = c("upgradeCost", "upgradeFactor", "climateFactor", 
                                            "maxDamage", "emergencyFactor"), 
                                   to = c("Upgrade Cost with", "Upgrade Amount with", "Climate Factor with", 
                                          "Resilience Factor with", "Emergency Factor with"))

bigMeans$strat <- mapvalues(bigMeans$strat, from = c("anticipate", "EE", "normal", "upgrade"),
                            to = c("Anticipatory", "Reactive", "Nominal", "Concurrent"))
setnames(bigMeans, "strat", "Strategy:")
bigMeans$sensVariable <- factor(bigMeans$sensVariable, levels = c("Upgrade Cost with", "Upgrade Amount with",  
                                                                  "Resilience Factor with", "Emergency Factor with",
                                                                  "Climate Factor with"))
bigMeans$climate <- factor(bigMeans$climate, levels = c("high rate of climate change", "low rate of climate change", "no climate change", "variable climate change"))



##############Create MultiNom Models#########################
##Construct Models
longAggMulti <- multinom(stratChoice~emergencyF + upgradeC + upgradeF + climateF + maxD + emergencyF*climateF + 
                           upgradeC * climateF + upgradeF * climateF, data = tcLongAgg)
longAggMulti <- stepAIC(longAggMulti, direction = "both")

longAggMultiNC <- multinom(stratChoice~emergencyF + upgradeC + upgradeF + maxD, data = tcLongAgg)

longAggMultiNC <- stepAIC(longAggMultiNC, direction = "both")

climateOnlyMulti <- multinom(stratChoice~climateF, data = tcLongAgg)
climateOnlyMulti <- stepAIC(climateOnlyMulti, direction = "both")


#################RPSS Junk#############################
##Test Agg Models
climo <- tcLongAgg[,.N/nrow(tcLongAgg), by = stratChoice]
climo <- setNames(climo$V1, climo$stratChoice)

yAggPred <- predict(longAggMulti, newdata = testLongAgg, type = "probs")
aggRPSS1 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = 1, "Upgrade" = 0, "React" = 0 ))
aggRPSS2 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = .5, "Upgrade" = .5, "React" = 0 ))
aggRPSS4 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = 0, "Upgrade" = 1, "React" = 0 ))
aggRPSS3 <- rps(testLongAgg$stratChoice, yAggPred, baseline = climo)

##Test Agg without CF
climo <- tcLongAgg[,.N/nrow(tcLongAgg), by = stratChoice]
climo <- setNames(climo$V1, climo$stratChoice)

yAggPred <- predict(longAggMultiNC, newdata = testLongAgg, type = "probs")
aggRPSSNC1 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = 1, "Upgrade" = 0, "React" = 0 ))
aggRPSSNC2 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = .5, "Upgrade" = .5, "React" = 0 ))
aggRPSSNC3 <- rps(testLongAgg$stratChoice, yAggPred, baseline = climo)

##Test Climate Only

climo <- tcLongAgg[,.N/nrow(tcLongAgg), by = stratChoice]
climo <- setNames(climo$V1, climo$stratChoice)

yAggPred <- predict(climateOnlyMulti, newdata = testLongAgg, type = "probs")
aggRPSSCO1 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = 1, "Upgrade" = 0, "React" = 0 ))
aggRPSSCO2 <- rps(testLongAgg$stratChoice, yAggPred, baseline = c("Nominal" = .5, "Upgrade" = .5, "React" = 0 ))
aggRPSSCO3 <- rps(testLongAgg$stratChoice, yAggPred, baseline = climo)


##########Create Combonations#########

load("climateDecrease.RData")
bigCulvert <- rbind(bigCulvert, bigCulvert_Decrease)
bigCulvert[, Iteration := unlist(lapply(1:936000, FUN = rep, 80))]



# Create Shuffle List for 99th%tile and 50th percentile
aggCulvertMed <- bigCulvert[variable == "SUM_totalCost",.(value = median(value)), by = .(culvertName, emergencyF, upgradeC, upgradeF, climateF, maxD, variable, strategy)]
aggCulvertMed <- matchList(aggCulvertMed, "SUM_totalCost", F)
aggCulvertMed[, iteration := unlist(lapply(1:2500, FUN = rep, 8))]
aggCulvertMed_Err <- createError(aggCulvertMed, "Median")

aggCulvert99 <- bigCulvert[,.(value = quantile(value, .99)), by = .(culvertName, emergencyF, upgradeC, upgradeF, climateF, maxD, variable, strategy)]
aggCulvert99[, iteration := unlist(lapply(1:2500, FUN = rep, 8))]
aggCulvert99 <- matchList(aggCulvert99, "SUM_totalCost", F)
aggCulvert99_Err <- createError(aggCulvert99, "99th Percentile")

aggCulvertMed_R <- shuffleCulverts(aggCulvertMed, longAggMultiNC, longAggMulti, 10, 1000, T)
aggCulvertMed <- shuffleCulverts(aggCulvertMed, longAggMultiNC, longAggMulti, 10, 1000)
aggCulvert99_R <- shuffleCulverts(aggCulvert99, longAggMultiNC, longAggMulti, 10, 1000, T)
aggCulvert99 <- shuffleCulverts(aggCulvert99, longAggMultiNC, longAggMulti, 10, 1000)

 
##Combine List
aggCulvertMed[, stat := "Median"]
aggCulvertMed_R[, stat := "Median"]
aggCulvert99[, stat := "99th Percentile"]
aggCulvert99_R[, stat := "99th Percentile"]
shuffleListR <- rbind(aggCulvertMed, aggCulvert99)
shuffleListRand <- rbind(aggCulvertMed_R, aggCulvert99_R)
##Calculate Percent Deviation from Best Strategy
shuffleListR <- shuffleListR[, lapply(.SD, function(x) (x-best)/best), by = .(climateBin, climateF, stat),
                            .SDcols = c("Nominal", "Upgrade", "best", "predicted", "climatePred")]
shuffleListRand <- shuffleListRand[, lapply(.SD, function(x) (x-best)/best), by = .(climateBin, climateF, stat),
                             .SDcols = c("best", "predicted")]

##Melt List
shuffleListR[, best := NULL]
shuffleListRand[, best := NULL]
setnames(shuffleListRand, "predicted", "Random")
shuffleListRand <- melt(shuffleListRand, id.vars = c(1:3))

shuffleListR <- melt(shuffleListR, id.vars = c(1:3))
shuffleListR <- rbind(shuffleListR, aggCulvertMed_Err, aggCulvert99_Err, shuffleListRand)

##Prep ShuffleList R for plotting
setnames(shuffleListR, "variable", "Strategy")
shuffleListR$Strategy <- mapvalues(shuffleListR$Strategy, 
                                   c("Upgrade", "predicted", "Nominal", "climatePred"), 
                                   c("Concurrent (Monolithic)", "Vertically Flexible", "Nominal (Monolithic)", "Vertically Flexible with\nClimate"))
shuffleListR[, stat := factor(stat, levels = c("Median", "99th Percentile"))]


ggplot(shuffleListR[(Strategy != "Vertically Flexible with\nClimate" & !grepl("Error", Strategy)) &
                      Strategy != "Random",], aes(x = climateF, y = value * 100, color = Strategy)) + 
  geom_smooth(method = "loess", level = .99, se = F) + facet_grid(.~stat) + 
  ylab("Percent Deviation from Lowest Cost Strategy") + 
  xlab("Climate Factor") + theme_gray(base_size = 11) + theme(legend.position = "bottom") +
  scale_color_brewer(palette = "Set1")
ggsave(file = "policyNC.jpg", width = 6.5, height = 5.5)


ggplot(shuffleListR[!grepl("Error", Strategy) & Strategy != "Random"], aes(x = climateF, y = value * 100, color = Strategy)) + 
  geom_smooth(method = "loess", level = .99, se = F) + facet_grid(.~stat) + 
  ylab("Percent Deviation from Lowest Cost Strategy") + 
  xlab("Climate Factor") + theme_gray(base_size = 11) + theme(legend.position = "bottom") +
  scale_color_brewer(palette = "Set1")
ggsave(file = "policy.jpg", width = 6.5, height = 5.5)

ggplot(shuffleListR, aes(x = climateF, y = value * 100, color = Strategy)) + 
  geom_smooth(method = "loess", level = .99, se = F) + facet_grid(.~stat) + 
  ylab("Percent Deviation from Lowest Cost Strategy") + 
  xlab("Climate Factor") + theme_gray(base_size = 11) + theme(legend.position = "bottom") + 
  scale_color_brewer(palette = "Set1")
ggsave(file = "verticalError.jpg", width = 6.5, height = 5.5)
  
ggsave(filename = "brewer_policy.jpg", width = 6.5, height = 5.5)

ggplot(shuffleListR, aes(x = climateF, y = value * 100, color = Strategy)) + 
  geom_smooth(method = "loess", se = F) + facet_grid(.~stat) + 
  ylab("Percent Deviation from Lowest Cost Strategy") + 
  theme(legend.position = "bottom") + xlab("Climate Factor") + theme_few(base_size = 11) +
  scale_color_few()
ggsave(filename = "few_policy.jpg", width = 6.5, height = 5.5)


bigList$strategy <- mapvalues(bigList$strategy, c("normal", "upgrade"), c("Nominal", "Concurrent"))
bigList <- bigList[variable == "SUM_totalCost" & strategy %in% c("Nominal", "Concurrent")]
setnames(bigList, "strategy", "Strategy")

bigList[, value := value/median(value)]

bigSample <- bigList[sample(.N, 6000)]

####Plot full results with smooth and scatter

###Plot with scatter
ggplot() + geom_smooth(data = bigList, aes(x = climateF, y = value, color = Strategy)) + 
  geom_point(data = bigSample, aes(x = climateF, y = value, color = Strategy), alpha = 1) + ylab("% of Median Cost") + 
  xlab("Climate Factor")
ggsave("resultsScatter.jpg")

ggplot() + geom_smooth(data = bigList, aes(x = climateF, y = value, color = Strategy)) + 
  geom_point(data = bigSample, aes(x = climateF, y = value, color = Strategy), alpha = 0) + ylab("% of Median Cost") + 
  xlab("Climate Factor")
ggsave("resultsSmooth.jpg")

ggplot(bigList[][sample(.N, 6000)], 
       aes(x = climateF, y = value, color = Strategy)) + geom_point()


ggplot(shuffleListR[sample(.N, 6000)], aes(x = climateF, y = value, color = variable)) + geom_jitter(width = .5) + facet_grid(.~stat)  


ggplot(shuffleListRC[sample(.N, 5000)], aes(x = Nominal, y = Upgrade, color = Cpredict)) + geom_point()
ggplot(shuffleListRC[sample(.N, 8000)], aes(x = Nominal, y = Upgrade, color = stratChoice)) + geom_point(alpha = .4)











#############################################################################################
#############################################################################################
#############################################################################################
###############################Code Not Used for Rotterdam###################################


shuffleListT <- resultsTable(shuffleListR, c(bestColumns, "climateDiff"), predColumns)
shuffleListT_Err <- resultsTable(shuffleListR, bestColumns, predColumns)


##Create Uncertainity
myRange <- shuffleList[, lapply(.SD, range), .SD = names(shuffleList)[1:5]]
myRange$climateF <- 0

uncertain <- seq(.1, .3, .5)
errorPreds <- data.frame(matrix(nrow = 3000, ncol = 3))
for(i in 1:3){
  rangeVector <- myRange[2] - myRange[1]
  rangeVector <- rangeVector * uncertain[i]
  error <- apply(rangeVector, 2, FUN = function(x) rnorm(24000,0,x))
  test <- shuffleList
  test[, 1:5] <- test[, 1:5, with = F] + error
  shuffleListR <- multiPredResults(longAggMultiNC, test, "C")
  shuffleListR <- shuffleListR[, .(Nominal =  sum(Nominal), Upgrade = sum(Upgrade), React = sum(React), 
                                   Anticipate = sum(Anticipate), Cacc = mean(Cacc), best = sum(best), 
                                   predicted = sum(predicted), CdiffBestN = sum(CdiffBestN), 
                                   CdiffPredN = sum(CdiffPredN),CdiffBestU = sum(CdiffBestU), 
                                   CdiffPredU = sum(CdiffPredU)), by = .(iteration, climateBin)]
  errorPreds[,i] <- (shuffleListR$predicted - shuffleListR$best)/shuffleListR$predicted
  shuffleListTemp <- resultsTable(shuffleListR, bestColumns, predColumns)
  stratName <- paste0("Prediction_", uncertain[i])
  shuffleListTemp <- shuffleListTemp[Strategy == "Model_Prediction",]
  shuffleListTemp[,Strategy := stratName]
  shuffleListT_Err <- rbind(shuffleListT_Err, shuffleListTemp)
}



randomResults <- multiPredResults(longAggMultiNC, shuffleList, "C")
randomResults[, predicted := ifelse(sample(c(0,1), 24000, T) == 1, Nominal, Upgrade)]
randomResults <- randomResults[, .(Nominal =  sum(Nominal), Upgrade = sum(Upgrade), React = sum(React), 
                                 Anticipate = sum(Anticipate), Cacc = mean(Cacc), best = sum(best), 
                                 predicted = sum(predicted), CdiffBestN = sum(CdiffBestN), 
                                 CdiffPredN = sum(CdiffPredN),CdiffBestU = sum(CdiffBestU), 
                                 CdiffPredU = sum(CdiffPredU)), by = .(iteration, climateBin)]
randomTable <- resultsTable(randomResults, bestColumns, predColumns)
randomTable[Strategy == "Model_Prediction", Strategy := "Random"]
shuffleListT_Err <- rbind(shuffleListT_Err, randomTable[Strategy == "Random"])




##Plot ShuffleListT





setnames(shuffleListT_Err, c("Mean_vs_Best", "x90_vs_Best"), c("Mean", "90th Percentile"))
shuffleListT_Err$Strategy <- mapvalues(shuffleListT_Err$Strategy, 
                                   c("All_Upgrade", "Model_Prediction", "Nominal", paste0("Prediction_",uncertain)), 
                                   c("Concurrent Strategy", "Vertically Flexible Strategy", "Nominal Strategy", paste0("Vertical Error ", uncertain)))
shuffleListT_Err$climate <- unlist(lapply(strsplit(
  substr(as.character(shuffleListT_Err$climate), 2, nchar(as.character(shuffleListT_Err$climate)) - 1),
  ","), function(x) median(as.numeric(x))))
shuffleListT_Err$climate <- mapvalues(shuffleListT_Err$climate, c(2.8850, shuffleListT_Err$climate[1]), c(3, .75))
shuffleListT_Err[, error := ifelse(grepl("Error", Strategy), "Error", 
                                   ifelse(grepl("Random", Strategy), "Random", "Normal"))]

shuffleListT_Err$error <- factor(shuffleListT_Err$error, levels = c("Normal", "Error", "Random"))
shuffleListT_Err[, Strategy := factor(Strategy, levels = c("Nominal Strategy", "Vertical Error 0.1", "Concurrent Strategy",
                                                       "Vertical Error 0.3", "Vertically Flexible Strategy", "Vertical Error 0.5", "Random" ) )]
####Create Shuffle Box and Whisker


shuffleListT2 <- shuffleListR[, .(upgradeDist = (Upgrade - best)/best, nomDist = (Nominal - best)/best,
                                  predDist = (predicted - best)/best, climate = climateBin)]
shuffleListT2 <- melt(shuffleListT2, 4)


climateText <- as.character(shuffleListR$climateBin)
climateText <- substr(climateText, 2, nchar(climateText) - 1)
climateText <- strsplit(climateText, ",")
climateText <- lapply(climateText, FUN = function(x) sapply(x, FUN = function(y) round(as.numeric(y), 2)))
climateText <- lapply(climateText, paste, collapse = "-")
climateText <- unlist(climateText)
climateText <- paste("Climate:", climateText)
shuffleListR[, climateBin := climateText]






#####Create Exemplar Graphs#######
load("emergencyFactor1.5/normal_normal.RData")
normal_normal <- data.table(normal_normal)
normalExample <- normal_normal[1:800 & culvertName == "Jackson",]
normalExample[, culvertName := "Nominal Strategy with Normal Climate"]
normalExample <- normalExample[1:100,]
normalExample <- melt(normalExample, c(1,5,7))
normalExample[, variable := mapvalues(variable, c("floodDamage", "construction"), c("Flood Damage", "Construction Cost"))]
normalExample[, value := value/1e4]
rm(normal_normal)

load("emergencyFactor1.5/high_normal.RData")
high_normal <- data.table(high_normal)
highExample <- high_normal[culvertName == "Jackson"]
highExample[, culvertName := "Nominal Strategy with High Climate"]
highExample <- highExample[701:800,]
highExample <- melt(highExample, c(1,5,7))
highExample[, variable := mapvalues(variable, c("floodDamage", "construction"), c("Flood Damage", "Construction Cost"))]
highExample[, value := value/1e4]
rm(high_normal)

load("emergencyFactor1.5/high_upgrade.RData")
high_upgrade <- data.table(high_upgrade)
i <- 1; j <- 100
i <- i + 100; j <- j + 100
upgradeExample <- high_upgrade[culvertName == "Jackson"]
upgradeExample[, culvertName := "Concurrent Strategy with High Climate"]
upgradeExample <- upgradeExample[901:1000,]
upgradeExample <- melt(upgradeExample, c(1,5,7))
upgradeExample[, variable := mapvalues(variable, c("floodDamage", "construction"), c("Flood Damage", "Construction Cost"))]
upgradeExample[, value := value/1e4]
rm(high_upgrade)

plotFrame <- rbind(normalExample, highExample, upgradeExample)
plotFrame$culvertName <- factor(plotFrame$culvertName, levels = c("Nominal Strategy with Normal Climate", "Nominal Strategy with High Climate", 
                                                                  "Concurrent Strategy with High Climate"))



####Write Tables

RPSS_table <- data.frame("Simulation" = c("Model with CF", "Model w/o CF", "Univariate Climate Model" ), 
                         "RPSS-Nominal" = sapply(c(aggRPSS1$rpss, aggRPSSNC1$rpss, aggRPSSCO1$rpss), round, 2), 
                         "RPSS-Nominal/Upgrade" = sapply(c(aggRPSS2$rpss, aggRPSSNC2$rpss, aggRPSSCO2$rpss), round, 2),
                         "RPSS-Climotology" = sapply(c(aggRPSS3$rpss, aggRPSSNC3$rpss, aggRPSSCO3$rpss), round, 2))

write.csv(RPSS_table, file = "RPSS.csv")




####Paper Outputs




##Local Sensitivity Analysis
ggplot(bigMeans[variable == "SUM_totalCost"], aes(x = sensValue, color = `Strategy:`)) + 
  geom_line(aes(y = myMean)) +
  facet_wrap(~sensVariable + climate, scales = "free_x", ncol = 5) + 
  ylab("Mean Cost in Millions of Dollars") + xlab("Sensitivty Values") + theme_few(base_size = 12) + 
  theme(legend.position = "bottom")
ggsave(filename = "completeMeans.jpg", width = 13, height = 6)


ggplot(bigMeans[variable == "SUM_totalCost" & sensVariable %in% c("Upgrade Cost with", "Emergency Factor with"),], aes(x = sensValue, color = `Strategy:`)) + 
  geom_line(aes(y = myMean)) +
  facet_wrap(~sensVariable + climate, scales = "free_x", ncol = 2) + 
  ylab("Mean Cost in Millions of Dollars") + xlab("Sensitivty Values") + theme_few(base_size = 12) + 
  theme(legend.position = "bottom")
ggsave(filename = "means1.jpg", width = 6.5, height = 9)


ggplot(bigMeans[variable == "SUM_totalCost" & sensVariable %in% c("Climate Factor with", "Resilience Factor with"),], aes(x = sensValue, color = `Strategy:`)) + 
  geom_line(aes(y = myMean)) +
  facet_wrap(~sensVariable + climate, scales = "free_x", ncol = 2) + 
  ylab("Mean Cost in Millions of Dollars") + xlab("Sensitivty Values") + theme_few(base_size = 12) + 
  theme(legend.position = "bottom")
ggsave(filename = "means2.jpg", width = 6.5, height = (9.2*.6666))

ggplot(bigMeans[variable == "SUM_delayHours" & sensVariable %in% 
                  c("Climate Factor with", "Resilience Factor with", "Upgrade Amount with") ,]) + 
  geom_line(aes(y = myMean, x = sensValue, color = `Strategy:`)) +
  facet_wrap(~sensVariable + climate, scales = "free_x") + 
  ylab("Mean Cost in Millions of Dollars") + xlab("Sensitivty Values") + theme_few() + theme(legend.position = "bottom")
ggsave(filename = "delayMeans.jpg", width = 6.5, height = (9.2*.6666))

ggplot(melt(shuffleListT_Err, c(1,6,7))[variable %in% c("Mean", "90th Percentile"), ], 
       aes(x = as.numeric(climate), y = value * 100)) + 
  geom_line(aes(color = Strategy, linetype = error)) + facet_grid(.~variable) + theme_few(base_size = 11) + xlab("Climate Factor") +
  ylab("Percent Deviation from Best Strategy") + theme(legend.position = "bottom") + scale_color_few() + 
  scale_linetype_manual(values = c("solid", "dashed", "dotted"), guide = F) + expand_limits(y=0)
ggsave(filename = "errorPolicy.jpg", width = 6.5, height = 5.5)


ggplot(shuffleListR[!grepl("Error", Strategy)], aes(x = Strategy, y = value * 100)) + geom_boxplot(aes(color = Strategy)) + facet_wrap(~climateBin) + theme_few(base_size = 11) + 
  theme(legend.position = "bottom", axis.ticks.x = element_blank(), axis.text.x = element_blank(), axis.title.x = element_blank()) + 
  ylab("Percent Deviation from Best Strategy") + scale_color_brewer(palette = "Set1")
ggsave(filename = "boxWhisker.jpg", width = 7, height = 6)

ggplot(plotFrame[variable %in% c("Flood Damage", "Construction Cost")], aes(x = year)) + geom_bar(aes(y = value, fill = variable), stat = "identity") + 
  ylab("Cost in Dollars (10s of Thousands)") + xlab("Year") + theme_few(base_size = 11) +
  facet_wrap(~culvertName, ncol = 1) + theme(legend.title = element_blank()) + scale_y_continuous(expand = c(0, 0), limits = c(0, 75)) 
ggsave(filename = "exemplar.jpg", width = 6.5, height = 5)
###Histogram and ECDF of big List
bigListPlot(bigList[variable %in% c("SUM_totalCost", "SUM_floodDamage", "SUM_construction") & strategy != "anticipate"], "histogram")
bigListPlot(bigList[variable %in% c("SUM_totalCost", "SUM_floodDamage", "SUM_construction") & strategy != "anticipate"], "cdf")
dev.off()

####Plot GEVs

min(getLocation("high", list(loc = 171.6821, scale = 56.868, shape = -.0123), designValue = 100, 
                  100))
loc = 171.6821, scale = 56.868, shape = -.0123

normalGEV <- rgev(n = 1000000, loc = 171.6821, scale = 56.868, shape = -.0123)
highGEV <- rgev(n = 1000000, loc = 208.9073, scale = 56.868, shape = -.0123)
normalDF <- data.frame(value = normalGEV, Climate = "Normal")
highDF <- data.frame(value = highGEV, Climate = "High")
plotDists <- rbind(normalDF, highDF)

ggplot(data = plotDists, aes(value, fill = Climate)) + geom_density(alpha = .5)


x1 <- seq(0,750, length.out = 10000)
densityDF <- data.frame(value = c(dgev(x1, loc = 171.6821, scale = 56.868, shape = -.0123), 
                                  dgev(x1, loc = 208.9073, scale = 56.868, shape = -.0123)),
                        Climate = c(rep("No Change", 10000), rep("High", 10000)), 
                        x = rep(x1, 2))
ggplot(densityDF, aes(x, value, fill = Climate)) + geom_polygon(alpha = .5) + geom_line(alpha = .75) + 
  theme_few(base_size = 11) + theme(legend.position = "bottom", axis.title.y = element_blank()) + 
  xlab("Event Magnitude") + ggtitle("Examples of Probability Density Graphs for\nGeneralized Extreme Value (GEV)\nDistributions")
ggsave(filename = "gevDist.jpg", width = 6.5, height = 5.5)

x2 <- seq(400,750, length.out = 10000)
probDF <- data.frame(value = c(pgev(x2, loc = 171.6821, scale = 56.868, shape = -.0123), 
                               pgev(x2, loc = 208.9073, scale = 56.868, shape = -.0123)),
                     Climate = c(rep("Normal", 10000), rep("High", 10000)), 
                     x = rep(x2, 2))
ggplot(probDF, aes(x, value, color = Climate)) + geom_line()



